package org.example;

import java.sql.*;

public class SQLite_java {
    private Connection conn;
    public void connect(){
        disconnect();
        try {
            Class.forName("org.sqlite.JDBC");
            conn = DriverManager.getConnection("jdbc:sqlite:Banka_autoincrement.db");
        } catch (ClassNotFoundException | SQLException e) {
            e.printStackTrace();
        }
    }

    public void disconnect(){
        if(conn != null) {
            try {
                conn.close();
                conn = null;
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    public void printAllUsers() {
        String sql = "select * from Komitent";
        try (Statement stmt = conn.createStatement();
             ResultSet rs = stmt.executeQuery(sql)){

            System.out.println("All users");
            while(rs.next()) {
                System.out.println(rs.getInt(1) +
                        "\t" + rs.getString(2) +
                        "\t" + rs.getString("Adresa"));
            }
        }  catch (SQLException e) {
            e.printStackTrace();
        }
    }


    public void printAllUsersWithName(String name) {
        String sql = "select * from Komitent where Naziv = ?";
        try (PreparedStatement ps = conn.prepareStatement(sql)) {
            ps.setString(1, name);
            try (ResultSet rs = ps.executeQuery()) {
                System.out.println("All users with name "+ name);
                while(rs.next()){
                    System.out.println(rs.getInt(1) +
                            "\t" + rs.getString(2) +
                            "\t" + rs.getString("Adresa"));
                }
            }  catch (SQLException e) {
                e.printStackTrace();
            }
        }  catch (SQLException e) {
            e.printStackTrace();
        }
    }

    public void printAllBankAccounts() {
        String sql = "select * from Racun";
        try (Statement stmt = conn.createStatement();) {
            try(ResultSet rs = stmt.executeQuery(sql)){
                System.out.println("All bank accounts");
                while(rs.next()){
                    System.out.println(rs.getInt("IdRac") +
                            "\t" + rs.getString("IdKom") +
                            "\t" + rs.getInt("Stanje"));
                }
            }  catch (SQLException e) {
                e.printStackTrace();
            }
        }  catch (SQLException e) {
            e.printStackTrace();
        }
    }

    public boolean updateUserAddress(int idKom, String address) {
        String sql = "update Komitent set Adresa = ? where idKom = ?";
        try (PreparedStatement ps = conn.prepareStatement(sql)) {
            ps.setInt(2, idKom);
            ps.setString(1, address);
            return ps.executeUpdate() > 0;
        }  catch (SQLException e) {
            e.printStackTrace();
        }
        return false;
    }

    public int insertUser(String name, String address) {
        String sql = "insert into Komitent (Naziv, Adresa) values (?, ?); ";
        try (PreparedStatement ps = conn.prepareStatement(sql)) {
            ps.setString(1, name);
            ps.setString(2, address);
            if(ps.executeUpdate() > 0) {
                try(ResultSet rs = ps.getGeneratedKeys()){
                    if(rs.next()) {
                        int id  = rs.getInt(1);
                        System.out.println("Dodat je user sa id " + id);
                        return id;
                    }
                }  catch (SQLException e) {
                    e.printStackTrace();
                }
            }

        }  catch (SQLException e) {
            e.printStackTrace();
        }
        return -1;
    }

    public void makeTransfer(int idRacFrom, int idRacTo, int iznos, boolean error) {
        String sql = "update Racun set Stanje = Stanje  + ? where IdRac = ?";
        try(PreparedStatement ps = conn.prepareStatement(sql)){

            conn.setAutoCommit(false);

            ps.setInt(2, idRacFrom);
            ps.setInt( 1, -iznos);
            ps.executeUpdate();

            if(error) {
                throw new Exception("Moja greska");
            }

            ps.setInt(2, idRacTo);
            ps.setInt( 1, iznos);
            ps.executeUpdate();

            conn.commit();
        }
        catch (Exception e) {
            try {
                conn.rollback();
            } catch (SQLException ex) {
                e.printStackTrace();
            }
        }
        finally {
            try {
                conn.setAutoCommit(true);
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    public void printAllUsersWithNameBAD(String name){
        String sql = "select * from Komitent where Naziv = '" + name + "'";
        try (Statement stmt = conn.createStatement();
         ResultSet rs = stmt.executeQuery(sql)) {
            System.out.println("BAD - All users with name "+ name);
            while(rs.next()){
                System.out.println(rs.getInt(1) +
                        "\t" + rs.getString(2) +
                        "\t" + rs.getString("Adresa"));
            }
        }  catch (SQLException e) {
            e.printStackTrace();
        }
    }
}

